﻿{ Network Driver Unit - Wireless

  Copyright (C) 2015 Maciej Kaczkowski - maciej@keit.co

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
}
unit nduWireless;
{$mode objfpc}
{$h+}
interface

uses
  Windows, Registry, Classes, SysUtils, Controls, Dialogs, Graphics,
  nduWlanAPI, nduCType, nduWlanTypes, nduNtDDNdis, nduWinIoCtl;

const
  NDU_WLAN_REG_ND = '\SOFTWARE\Microsoft\Windows NT\CurrentVersion\NetworkCards';

type
  TnduWlanEvent = procedure(Sender: TObject) of object;

  TInterfaceItem = record
    Name: string;
    GUID: string;
  end;

  TWireless = class;

  { TNetworkItem }

  TNetworkItem = class(TObject)
  private
    FChannel: Integer;
    FParent: TWireless;
    function GetChannel: Integer;
    function GetChannelAsString: string;
    function GetChannelHalfWidth: Integer;
    function GetChannelWidth: Integer;
    function GetEncryption: string;
    function GetQuality: Integer;
  public
    SSID: string;
    Frequency: DWORD;
    RSSI: Integer;
    MAC: string;
    BSS: DWORD; // adhoc, infrastructure, auto
    Privacy: DWORD;
    FirstDetected,
    LastDetected: TDateTime;
    Items: TStrings;
    // HT
    Is40MHz: Boolean;
    ShortGi20MHz: Boolean;
    ShortGi40MHz: Boolean;
    SecondaryChannelLower: Boolean;
    constructor Create;
    destructor Destroy; override;
    property Channel: Integer read GetChannel;
    property ChannelWidth: Integer read GetChannelWidth;
    property ChannelHalfWidth: Integer read GetChannelHalfWidth;
    property ChannelAsString: string read GetChannelAsString;
    property Encryption: string read GetEncryption;
    property Quality: Integer read GetQuality;
    property Parent: TWireless read FParent write FParent;

    function BSSAsString: string;
  end;

  { TWireless }

  TWireless = class(TObject)
  protected
    FCurrentInterfaceHandle: Handle;
    FInterfaces: TStrings;
    FInterfacesIndex: Integer;
    FNetworkList: TStrings;
    FNetworkIndex: Integer;
    FLastNetworkScan: TDateTime;
    function GetNetworkCount: Integer; virtual;
    function GetNetworkItem(Index: Integer): TNetworkItem; virtual;
    function GetNetworkItemByMAC(MAC: string): TNetworkItem; virtual;
    function GetInterfaces(Index: Integer): TInterfaceItem; virtual;
    procedure SetInterfacesIndex(AValue: Integer); virtual;

    procedure ParseHTCap(net: TNetworkItem; data: string);
    procedure ParseHTOper(net: TNetworkItem; data: string);
    procedure ParseIE(net: TNetworkItem; data: string; idx: Integer);
  public
    constructor Create; virtual;
    destructor Destroy; override;

    procedure InterfacesRefresh; virtual;
    function InterfacesCount: Integer;
    property Interfaces[Index: Integer]: TInterfaceItem read GetInterfaces;
    property InterfacesIndex: Integer read FInterfacesIndex write SetInterfacesIndex;

    property NetworkCount: Integer read GetNetworkCount;
    property NetworkItem[Index: Integer]: TNetworkItem read GetNetworkItem;
    property NetworkIndex: Integer read FNetworkIndex write FNetworkIndex;

    function NetworkScan: Boolean; virtual;
    function NetworkForceScan: Boolean; virtual;

    property LastNetworkScan: TDateTime read FLastNetworkScan;
  end;

  { TWirelessNDIS }

  TWirelessNDIS = class(TWireless)
  protected
    procedure SetInterfacesIndex(AValue: Integer); override;
  public
    procedure InterfacesRefresh; override;
    function NetworkScan: Boolean; override;
    function NetworkForceScan: Boolean; override;
  end;

  { TWirelessWLAN }

  TWirelessWLAN = class(TWireless)
  protected
    FGUID: TGUID;
    FLastError: DWORD;
    FPHandle: THandle;
    procedure SetInterfacesIndex(AValue: Integer); override;
  public
    constructor Create; override;
    destructor Destroy; override;
    procedure InterfacesRefresh; override;
    function NetworkScan: Boolean; override;
    function NetworkForceScan: Boolean; override;
  end;

function CreateWirelessDriver: TWireless;

function EIDToString(x: DWORD): string;

implementation

{ direct copy from --rtutils.pas-- }
function StrBefore(str, str1: string; CaseSensitive: Boolean = True): string;
var
  i: integer;
begin
  if CaseSensitive then
    i := Pos(str1, str)
  else
    i := Pos(LowerCase(str1), LowerCase(str));
  Result := Copy(str, 1, i - 1);
end;

function StrAfter(str, str1: string; CaseSensitive: Boolean = True): string;
var
  i: integer;
begin
  if CaseSensitive then
    i := Pos(str1, str)
  else
    i := Pos(LowerCase(str1), LowerCase(str));
  Result := Copy(str, i + Length(str1), Length(str));
end;

function StringToHexString(s: string; separator: string = ':'): string;
var
  i, j: Integer;
begin
  Result := '';
  j := Length(s);
  for i := 1 to j do
  begin
    Result := Result + Format('%.2x', [byte(s[i])]);
    if i < j then
      Result := Result + separator;
  end;
end;

procedure FreeObjects(s: TStrings);
var
  i: Integer;
  o: TObject;
begin
  for i := 0 to s.Count - 1 do
  begin
    o := s.Objects[i];
    if Assigned(o) and (o <> nil) then
      FreeAndNil(o);
    s.Objects[i] := nil;
  end;
end;

{ direct copy from --rtutils.pas-- }

function CreateWirelessDriver: TWireless;
begin
  if SysUtils.Win32MajorVersion >= 6 then
    Result := TWirelessWLAN.Create
  else
    Result := TWirelessNDIS.Create;
end;

function EIDToString(x: DWORD): string;
begin
  case x of
    WLAN_EID_SSID:
      Result := 'SSID';
    WLAN_EID_SUPP_RATES:
      Result := 'SUPP_RATES';
    WLAN_EID_FH_PARAMS:
      Result := 'FH_PARAMS';
    WLAN_EID_DS_PARAMS:
      Result := 'DS_PARAMS';
    WLAN_EID_CF_PARAMS:
      Result := 'CF_PARAMS';
    WLAN_EID_TIM:
      Result := 'TIM';
    WLAN_EID_IBSS_PARAMS:
      Result := 'IBSS_PARAMS';
    WLAN_EID_COUNTRY:
      Result := 'COUNTRY';
    WLAN_EID_REQUEST:
      Result := 'REQUEST';
    WLAN_EID_CHALLENGE:
      Result := 'CHALLENGE';
    // EIDs defined by IEEE 802.11h - START */
    WLAN_EID_PWR_CONSTRAINT:
      Result := 'PWR_CONSTRAINT';
    WLAN_EID_PWR_CAPABILITY:
      Result := 'PWR_CAPABILITY';
    WLAN_EID_TPC_REQUEST:
      Result := 'TPC_REQUEST';
    WLAN_EID_TPC_REPORT:
      Result := 'TPC_REPORT';
    WLAN_EID_SUPPORTED_CHANNELS:
      Result := 'SUPPORTED_CHANNELS';
    WLAN_EID_CHANNEL_SWITCH:
      Result := 'CHANNEL_SWITCH';
    WLAN_EID_MEASURE_REQUEST:
      Result := 'MEASURE_REQUEST';
    WLAN_EID_MEASURE_REPORT:
      Result := 'MEASURE_REPORT';
    WLAN_EID_QUITE:
      Result := 'QUITE';
    WLAN_EID_IBSS_DFS:
      Result := 'IBSS_DFS';
    // EIDs defined by IEEE 802.11h - END */
    WLAN_EID_ERP_INFO:
      Result := 'ERP_INFO';
    WLAN_EID_HT_CAP:
      Result := 'HT_CAP';
    WLAN_EID_RSN:
      Result := 'RSN';
    WLAN_EID_EXT_SUPP_RATES:
      Result := 'EXT_SUPP_RATES';
    WLAN_EID_POWER_CONSTRAINT:
      Result := 'POWER_CONSTRAINT';
    WLAN_EID_MOBILITY_DOMAIN:
      Result := 'MOBILITY_DOMAIN';
    WLAN_EID_FAST_BSS_TRANSITION:
      Result := 'FAST_BSS_TRANSITION';
    WLAN_EID_TIMEOUT_INTERVAL:
      Result := 'TIMEOUT_INTERVAL';
    WLAN_EID_RIC_DATA:
      Result := 'RIC_DATA';
    WLAN_EID_HT_OPERATION:
      Result := 'HT_OPERATION';
    WLAN_EID_SECONDARY_CHANNEL_OFFSET:
      Result := 'SECONDARY_CHANNEL_OFFSET';
    WLAN_EID_20_40_BSS_COEXISTENCE:
      Result := '20_40_BSS_COEXISTENCE';
    WLAN_EID_20_40_BSS_INTOLERANT:
      Result := '20_40_BSS_INTOLERANT';
    WLAN_EID_OVERLAPPING_BSS_SCAN_PARAMS:
      Result := 'OVERLAPPING_BSS_SCAN_PARAMS';
    WLAN_EID_MMIE:
      Result := 'MMIE';
    WLAN_EID_VENDOR_SPECIFIC:
      Result := 'VENDOR_SPECIFIC';
  else
    Result := 'UNKNOWN';
  end;
end;

{ TWirelessWLAN }

procedure TWirelessWLAN.SetInterfacesIndex(AValue: Integer);
begin
  inherited SetInterfacesIndex(AValue);
  FGUID := StringToGUID(FInterfaces.Names[AValue]);
end;

constructor TWirelessWLAN.Create;
var
  dwVersion: DWORD;
begin
  inherited Create;
  FLastError := WlanOpenHandle(1, nil, @dwVersion, @FCurrentInterfaceHandle);
end;

destructor TWirelessWLAN.Destroy;
begin
  if @FCurrentInterfaceHandle <> nil then
    WlanCloseHandle(FCurrentInterfaceHandle, nil);
  inherited Destroy;
end;

procedure TWirelessWLAN.InterfacesRefresh;
var
  pInterface: Pndu_WLAN_INTERFACE_INFO_LIST;
  hClient: THandle;
  ResultInt: DWORD;
  i: Integer;
begin
  inherited InterfacesRefresh;
  if FLastError = ERROR_SUCCESS then
  begin
    ResultInt := WlanEnumInterfaces(FCurrentInterfaceHandle, nil, @pInterface);
    if ResultInt = ERROR_SUCCESS then
    begin
      for i := 0 to pInterface^.dwNumberOfItems - 1 do
      begin
        FInterfaces.Add(GUIDToString(pInterface^.InterfaceInfo[i].InterfaceGuid)+'='+
                        pInterface^.InterfaceInfo[i].strInterfaceDescription);
      end;
    end;
  end;
end;

function TWirelessWLAN.NetworkScan: Boolean;
var
  //wlan_bsslist: Pndu_WLAN_BSS_LIST;
  guid: PGUID;
  mac: string;
  tmp: string;
  net: TNetworkItem;
  i, j: Integer;
  data: Pointer;
  size: DWORD;
  m: TMemoryStream;
  rec: Tndu_WLAN_BSS_ENTRY;
  remPos: DWord;
  entPos: DWord;
  items: DWord;
begin
  m := TMemoryStream.Create;
  try
    guid := @FGUID;

    if WlanGetNetworkBssList(FCurrentInterfaceHandle, guid, nil, dot11_BSS_type_any,
      False, nil, @data) = ERROR_SUCCESS then
    begin
      Move(data^, size, 4);
      m.SetSize(size);
      Move(data^, m.Memory^, size);
      WlanFreeMemory(data);
    end;

    m.Position := 0;
    if m.Size > 8 then
    begin
      m.ReadDWord; // size of data
      items := m.ReadDWord;
      if items > 0 then
        inherited;

      for i := 0 to items - 1 do
      begin
        entPos := m.Position;
        m.Read(rec, SizeOf(rec));
        //ShowMessage(Format('%.6x:%.6x', [rec.ulIeOffset, rec.ulIeSize]));

        SetLength(mac, 6);
        Move(rec.dot11Bssid, mac[1], 6);
        mac := StringToHexString(mac, '');

        net := GetNetworkItemByMAC(mac);

        if rec.ulIeSize > 0 then
        begin
          remPos := m.Position;
          m.Position := entPos + rec.ulIeOffset; // WLAN_BSS_ENTRY + offset
          SetLength(tmp, rec.ulIeSize);
          m.Read(tmp[1], rec.ulIeSize);
          ParseIE(net, tmp, 1);
          m.Position := remPos;
        end;

        SetLength(net.SSID, rec.dot11Ssid.uSSIDLength);
        Move(rec.dot11Ssid.ucSSID, net.SSID[1], rec.dot11Ssid.uSSIDLength);

        net.Frequency := rec.ulChCenterFrequency div 1000;
        net.BSS := DWORD(rec.dot11BssType);
        net.RSSI := rec.lRssi;
        net.Privacy := Byte(rec.usCapabilityInformation and $10 = $10);
      end;
    end;
  finally
    m.Free;
  end;
end;

function TWirelessWLAN.NetworkForceScan: Boolean;
var
  guid: PGUID;
begin
  Result:=inherited NetworkForceScan;
  guid := @FGUID;
  WlanScan(FCurrentInterfaceHandle, guid, nil, nil, nil);
end;

{ TWireless }

procedure TWireless.SetInterfacesIndex(AValue: Integer);
begin
  if FInterfacesIndex = AValue then
    Exit;
  FInterfacesIndex := AValue;

  FreeObjects(FNetworkList);
  FNetworkList.Clear;
  FNetworkIndex := -1;
end;

procedure TWireless.ParseHTCap(net: TNetworkItem; data: string);
begin
  if Length(data) > 0 then
  begin
    net.Is40MHz := Byte(data[1]) and $02 = $02;
    net.ShortGi20MHz := Byte(data[1]) and $20 = $20;
    net.ShortGi40MHz := Byte(data[1]) and $40 = $40;
  end;
end;

procedure TWireless.ParseHTOper(net: TNetworkItem; data: string);
const
  channel = 1;
  subset1 = 2;
begin
 ////Primary channel
 //settings.PrimaryChannel = ie.ItsData[0];
 ////Secondary channel location
  net.SecondaryChannelLower := False;
  if Length(data) >= subset1 then
  begin
    net.SecondaryChannelLower := Byte(data[channel]) and $03 = $03;
    if net.Is40MHz then
      net.Is40MHz := (Byte(data[subset1]) and $03 = $03) or (Byte(data[subset1]) and $01 = $01);
  end;
end;

procedure TWireless.ParseIE(net: TNetworkItem; data: string; idx: Integer);
var
  k, j: Integer;
  str: string;
begin
  net.Items.Clear;
  while idx < Length(data) do
  begin
    k := Byte(data[idx + 0]); // element id
    j := Byte(data[idx + 1]); // length
    str := Copy(data, idx + 2, j);
    idx := idx + j + 2;
    net.Items.Add(IntToStr(k)+'='+StringToHexString(str, ''));
    case k of
      45: ParseHTCap(net, str);
      61: ParseHTOper(net, str);
    end;
  end;
end;

function TWireless.GetNetworkCount: Integer;
begin
  Result := FNetworkList.Count;
end;

function TWireless.GetNetworkItem(Index: Integer): TNetworkItem;
begin
  Result := TNetworkItem(FNetworkList.Objects[Index]);
end;

function TWireless.GetNetworkItemByMAC(MAC: string): TNetworkItem;
var
  idx: Integer;
begin
  idx := FNetworkList.IndexOf(mac);
  if idx = -1 then
  begin
    Result := TNetworkItem.Create;
    Result.FirstDetected := Now;
    Result.LastDetected := Now;
    Result.MAC := mac;
    Result.Parent := Self;
    FNetworkList.AddObject(mac, Result);
  end
  else
  begin
    Result := TNetworkItem(FNetworkList.Objects[idx]);
    Result.LastDetected := Now;
  end;
end;

constructor TWireless.Create;
begin
  FInterfaces := TStringList.Create;
  FNetworkList := TStringList.Create;
end;

destructor TWireless.Destroy;
begin
  FreeObjects(FNetworkList);
  FNetworkList.Free;
  FInterfaces.Free;
  inherited Destroy;
end;

function TWireless.GetInterfaces(Index: Integer): TInterfaceItem;
begin
  Result.Name := FInterfaces.ValueFromIndex[Index];
  Result.GUID := FInterfaces.Names[Index];
end;

function TWireless.InterfacesCount: Integer;
begin
  Result := FInterfaces.Count;
end;

function TWireless.NetworkScan: Boolean;
var
  i: Integer;
  net: TNetworkItem;
begin
  Result := False;
  FLastNetworkScan := Now;

  //for i := 0 to FNetworkList.Count - 1 do
  //begin
  //  net := TNetworkItem(FNetworkList.Objects[i]);
  //  net.RSSI := -100;
  //end;
end;

function TWireless.NetworkForceScan: Boolean;
begin

end;

procedure TWireless.InterfacesRefresh;
begin
  FInterfaces.Clear;
  FInterfacesIndex := -1;
  FreeObjects(FNetworkList);
  FNetworkList.Clear;
  FNetworkIndex := -1;
end;

{ TNetworkItem }

function TNetworkItem.GetChannel: Integer;
begin
  if (Frequency >= 2412) and (Frequency <= 2484) then
  begin
    Result := (Frequency - 2412) div 5 + 1;
    if Result = 15 then
      Result := 14;
  end
  else
  if (Frequency >= 5170) and (Frequency <= 5825) then
  begin
    Result := (Frequency - 5170) div 5 + 34;
  end
  else
  begin
    Result := -1;
  end;
end;

function TNetworkItem.GetChannelAsString: string;
const
  FMT = '%d + %d';
var
  ch: Integer;
begin
  ch := Channel;
  if Is40MHz then
    if SecondaryChannelLower then
      Result := Format(FMT, [Ch, Ch - 4])
    else
      Result := Format(FMT, [Ch, Ch + 4])
  else
    Result := Format('%d', [Ch]);
end;

function TNetworkItem.GetChannelHalfWidth: Integer;
begin
  Result := 20 div 2; // basic channel width
end;

function TNetworkItem.GetChannelWidth: Integer;
begin
  if Is40MHz then
    Result := 40
  else
    Result := 20;
end;

function TNetworkItem.GetEncryption: string;
var
  i: Integer;
  _name: string;
  _data: string;
  WPS: Boolean;
begin
  Result := 'Open';
  WPS := False;
  if Privacy <> 0 then
  begin
    Result := 'WEP';
    for i := 0 to Items.Count - 1 do
    begin
      _name := StrBefore(Items[i], '=');
      _data := StrAfter(Items[i], '=');
      if _name = IntToStr(WLAN_EID_RSN) then
      begin
        case Copy(_data, 1, 6*2) of
          '0100000FAC00': Result := 'GROUP';
          '0100000FAC01': Result := 'WEP-40';
          '0100000FAC02': Result := 'WPA/TKIP';
          '0100000FAC04': Result := 'WPA2/CCMP';
          '0100000FAC05': Result := 'WEP-104';
          //https://code.google.com/p/reaver-wps/source/browse/trunk/src/common/ieee802_11_defs.h?r=3
          '0100000FAC06': Result := 'AES/CMAC';
        else
          Result := 'Unknown';
        end
      end
      else
      if _name = IntToStr(WLAN_EID_VENDOR_SPECIFIC) then
      begin
        case Copy(_data, 1, 4*2) of
          // Microsoft
          '0050F201': Result := 'WPA-Personal';
          '0050F204': WPS := True;
        end
      end;
    end;

    if WPS then
      Result := Result + ' + WPS';
  end;
end;

function TNetworkItem.GetQuality: Integer;
begin
  if (RSSI <= -100) then
    Result := 0
  else
  if (RSSI >= -50) then
    Result := 100
  else
    Result := 2 * (RSSI + 100);
end;

constructor TNetworkItem.Create;
begin
  Items := TStringList.Create;
  FChannel := -1;
end;

destructor TNetworkItem.Destroy;
begin
  Items.Free;
  inherited Destroy;
end;

function TNetworkItem.BSSAsString: string;
begin
  Result := '';
  if Parent is TWirelessNDIS then
    case Tndu_NDIS_802_11_NETWORK_INFRASTRUCTURE(BSS) of
      Ndis802_11IBSS: Result := 'ad-hoc';
      Ndis802_11Infrastructure: Result := 'infrastructure';
      Ndis802_11AutoUnknown: Result := 'auto';
      Ndis802_11InfrastructureMax: Result := 'infrastructure';
    end
  else
  if Parent is TWirelessWLAN then
    case Tndu_DOT11_BSS_TYPE(BSS) of
      dot11_BSS_type_infrastructure: Result := 'infrastructure';
      dot11_BSS_type_independent: Result := 'ad-hoc';
      dot11_BSS_type_any: Result := 'auto';
    end;
end;

{ TWirelessNDIS }

procedure TWirelessNDIS.InterfacesRefresh;
var
  reg: TRegistry;
  str: TStrings;
  i: Integer;
begin
  inherited;
  reg := TRegistry.Create;
  str := TStringList.Create;
  try
    reg.RootKey := HKEY_LOCAL_MACHINE;
    if reg.OpenKeyReadOnly(NDU_WLAN_REG_ND) then
    begin
      reg.GetKeyNames(str);
      for i := 0 to str.Count - 1 do
      begin
        if reg.OpenKeyReadOnly(NDU_WLAN_REG_ND+'\'+str[i]) then
        begin
          if reg.ValueExists('ServiceName') and reg.ValueExists('Description') then
          begin
            FInterfaces.Add(reg.ReadString('ServiceName')+'='+reg.ReadString('Description'));
          end;
        end;
      end;
    end;
  finally
    str.Free;
    reg.Free;
  end;
end;

procedure TWirelessNDIS.SetInterfacesIndex(AValue: Integer);
var
  tmp: string;
begin
  inherited;

  tmp := '\\.\' + FInterfaces.Names[AValue];

  if (FCurrentInterfaceHandle <> INVALID_HANDLE_VALUE) then
    CloseHandle(FCurrentInterfaceHandle);

  FCurrentInterfaceHandle := CreateFileA(PChar(tmp), GENERIC_READ, FILE_SHARE_READ or
    FILE_SHARE_WRITE, nil, OPEN_EXISTING, 0, 0);

  //IsOpened := False;
  //if FCurrentInterfaceHandle <> INVALID_HANDLE_VALUE then
  //begin
  //  IsOpened := True;
  //end;
end;

function TWirelessNDIS.NetworkScan: Boolean;
const
  BUF_SIZE = 65536;
var
  oidcode: ulong;
  BytesReturned: ulong;
  i, j: integer;
  tmp: string;
  data: Pointer;
  m: TMemoryStream;
  rec: Tndu_NDIS_WLAN_BSSID_EX;
  net: TNetworkItem;
  mac: string;
  nextpos: DWORD;
begin
  Result := inherited;

  data := GetMem(BUF_SIZE);
  if data = nil then
    Exit;

  NetworkForceScan;

  // get list

  oidcode := NDU_OID_802_11_BSSID_LIST;

  m := TMemoryStream.Create;
  if DeviceIoControl(FCurrentInterfaceHandle, NDIS_CONTROL_CODE(0, NDU_METHOD_OUT_DIRECT),
    @oidcode, SizeOf(oidcode), data, BUF_SIZE, BytesReturned, nil) then
  begin
    m.SetSize(BytesReturned);
    Move(data^, m.Memory^, BytesReturned)
  end;

  FreeMem(data, BUF_SIZE);

  m.Position := 0;
  if m.Size > 4 then
  begin
    for i := 0 to m.ReadDWord - 1 do
    begin
      rec.Length := m.ReadDWord;
      m.Position := m.Position - SizeOf(DWORD);

      if rec.Length < 120 then
      begin
        m.Read(rec, SizeOf(Tndu_NDIS_WLAN_BSSID));// odczytaj
        m.Position := m.Position - SizeOf(Tndu_NDIS_WLAN_BSSID); // wycofaj
        rec.IELength := 0;
        for j := 8 to High(rec.SupportedRates) do
          rec.SupportedRates[j] := 0;
      end
      else
      begin
        m.Read(rec, SizeOf(Tndu_NDIS_WLAN_BSSID_EX)); // odczytaj ex
        m.Position := m.Position - SizeOf(Tndu_NDIS_WLAN_BSSID_EX); // wycofaj ex
      end;

      nextpos := m.Position + rec.Length;

      SetLength(mac, 6);
      Move(rec.MacAddress, mac[1], 6);
      mac := StringToHexString(mac, '');

      // pobierz lub utworz "net"
      net := GetNetworkItemByMAC(mac);

      if rec.IELength > 0 then
      begin
        m.Position := m.Position + // aktualna pozycja
          SizeOf(Tndu_NDIS_WLAN_BSSID) + // rozmiar starego
          SizeOf(Tndu_NDIS_802_11_RATES_EX) - SizeOf(Tndu_NDIS_802_11_RATES) +
            //(z SupportedRates 8, ex ma 16 więc różnica, aby dotrzeć do IELength)
          SizeOf(rec.IELength); // pomin IELength bo juz jest odczytane
        SetLength(tmp, rec.IELength);
        m.Read(tmp[1], rec.IELength);
        ParseIE(net, tmp, 1+SizeOf(Tndu_NDIS_802_11_FIXED_IEs));// pomiń FIXED_IEs
      end;

      SetLength(net.SSID, rec.Ssid.SsidLength);
      Move(rec.Ssid.Ssid, net.SSID[1], rec.Ssid.SsidLength);

      net.Frequency := rec.Configuration.DSConfig div 1000;
      //if FNetworkList.Count = 1 then
        //net.Frequency := 2472; - 13ch
        //net.Frequency := 2484; - 14ch
        //net.Frequency := 5700; - 140ch
        //net.Frequency := 5180; - 36ch
      net.BSS := DWORD(rec.InfrastructureMode);
      net.RSSI := rec.Rssi;
      net.Privacy := rec.Privacy;

      m.Position := nextpos;
    end;
  end;

  m.Free;
end;

function TWirelessNDIS.NetworkForceScan: Boolean;
var
  oidcode: ulong;
  BytesReturned: ulong;
begin
  Result := inherited NetworkForceScan;

  oidcode := NDU_OID_802_11_BSSID_LIST_SCAN;
  DeviceIoControl(FCurrentInterfaceHandle, NDIS_CONTROL_CODE(0, NDU_METHOD_OUT_DIRECT),
    @oidcode, SizeOf(oidcode), nil, 0, BytesReturned, nil);

  Sleep(200);
end;

end.
